@@ -74,8 +74,10 @@ tree_detection = function(x, ws, hmin = 2)
 #'@export
 tree_detection.LAS = function(x, ws, hmin = 2)
 {
-  if (ws <= 0) stop("ws should be stricly positive", call. = FALSE)
-  if (hmin < 0) stop("hmin should be positive", call. = FALSE)
+  assertive::assert_is_a_number(ws)
+  assertive::assert_all_are_positive(ws)
+  assertive::assert_is_a_number(hmin)
+  assertive::assert_all_are_positive(hmin)
 
   . <- X <- Y <- Z <- NULL
   maxima = C_LocalMaximaPoints(x, ws, hmin)
@@ -85,6 +87,11 @@ tree_detection.LAS = function(x, ws, hmin = 2)
 #'@export
 tree_detection.lasmetrics = function(x, ws, hmin = 2)
 {
+  assertive::assert_is_a_number(ws)
+  assertive::assert_all_are_positive(ws)
+  assertive::assert_is_a_number(hmin)
+  assertive::assert_all_are_positive(hmin)
+
   x = as.raster(x)
   return(tree_detection(x, ws, hmin))
 }
@@ -92,6 +99,11 @@ tree_detection.lasmetrics = function(x, ws, hmin = 2)
 #'@export
 tree_detection.RasterLayer = function(x, ws, hmin = 2)
 {
+  assertive::assert_is_a_number(ws)
+  assertive::assert_all_are_positive(ws)
+  assertive::assert_is_a_number(hmin)
+  assertive::assert_all_are_positive(hmin)
+
   xx <- raster::as.matrix(x)
   xx <- t(apply(xx, 2, rev))
   LM = tree_detection(xx, ws, hmin)
@@ -103,11 +115,9 @@ tree_detection.RasterLayer = function(x, ws, hmin = 2)
 #'@export
 tree_detection.matrix = function(x, ws, hmin = 2)
 {
-  if (ws < 3)
-    stop("ws should be equal or greater than 3", call. = FALSE)
-
-  if (ws %% 2 == 0)
-    stop("ws should be an odd number", call. = FALSE)
+  assertive::assert_is_a_number(ws)
+  assertive::assert_all_are_greater_than_or_equal_to(ws, 3)
+  assertive::assert_all_are_odd(ws)
 
   x[is.na(x)] <- -Inf
   LM = C_LocalMaximaMatrix(x, ws, hmin)
