@@ -52,12 +52,4 @@ polygon_area = function(x, y)
 	area  = abs(area*0.5)
 
 	return(area)
-}
-
-area = function(x, y)
-{
-  hull = convex_hull(x, y)
-  area = polygon_area(hull$x, hull$y)
-  area = round(area,1)
-  return(area)
 }
\ No newline at end of file
